@@ -37,15 +37,7 @@
 
 #include <stdint.h>
 #include <stddef.h>
-#if defined(_MSC_VER) && (_MSC_VER < 1900)
-/** VS2010 does not support inttypes - remove when VS2010 support is dropped */
-#define __PRI64_PREFIX "ll"
-#define PRIX64 __PRI64_PREFIX "X"
-#define PRIu64 __PRI64_PREFIX "u"
-#define PRIu32 "u"
-#else
 #include <inttypes.h>
-#endif
 
 
 /** @defgroup ocsd_interfaces OpenCSD Library : Interfaces
@@ -85,61 +77,64 @@ typedef uint32_t ocsd_trc_index_t;   /**< Trace source index type - 32 bit size
 typedef enum _ocsd_err_t {
 
     /* general return errors */
-    OCSD_OK = 0,                   /**< No Error. */
-    OCSD_ERR_FAIL,                 /**< General systemic failure. */
-    OCSD_ERR_MEM,                  /**< Internal memory allocation error. */
-    OCSD_ERR_NOT_INIT,             /**< Component not initialised or initialisation failure. */
-    OCSD_ERR_INVALID_ID,           /**< Invalid CoreSight Trace Source ID.  */
-    OCSD_ERR_BAD_HANDLE,           /**< Invalid handle passed to component. */
-    OCSD_ERR_INVALID_PARAM_VAL,    /**< Invalid value parameter passed to component. */
-    OCSD_ERR_INVALID_PARAM_TYPE,   /**< Type mismatch on abstract interface */
-    OCSD_ERR_FILE_ERROR,           /**< File access error */
-    OCSD_ERR_NO_PROTOCOL,          /**< Trace protocol unsupported */
+    OCSD_OK = 0,                   /**< 0 No Error. */
+    OCSD_ERR_FAIL,                 /**< 1 General systemic failure. */
+    OCSD_ERR_MEM,                  /**< 2 Internal memory allocation error. */
+    OCSD_ERR_NOT_INIT,             /**< 3 Component not initialised or initialisation failure. */
+    OCSD_ERR_INVALID_ID,           /**< 4 Invalid CoreSight Trace Source ID.  */
+    OCSD_ERR_BAD_HANDLE,           /**< 5 Invalid handle passed to component. */
+    OCSD_ERR_INVALID_PARAM_VAL,    /**< 6 Invalid value parameter passed to component. */
+    OCSD_ERR_INVALID_PARAM_TYPE,   /**< 7 Type mismatch on abstract interface */
+    OCSD_ERR_FILE_ERROR,           /**< 8 File access error */
+    OCSD_ERR_NO_PROTOCOL,          /**< 9 Trace protocol unsupported */
     /* attachment point errors */
-    OCSD_ERR_ATTACH_TOO_MANY,      /**< Cannot attach - attach device limit reached. */
-    OCSD_ERR_ATTACH_INVALID_PARAM, /**< Cannot attach - invalid parameter. */
-    OCSD_ERR_ATTACH_COMP_NOT_FOUND,/**< Cannot detach - component not found. */
+    OCSD_ERR_ATTACH_TOO_MANY,      /**< 10 Cannot attach - attach device limit reached. */
+    OCSD_ERR_ATTACH_INVALID_PARAM, /**< 11 Cannot attach - invalid parameter. */
+    OCSD_ERR_ATTACH_COMP_NOT_FOUND,/**< 12 Cannot detach - component not found. */
     /* source reader errors */
-    OCSD_ERR_RDR_FILE_NOT_FOUND,   /**< source reader - file not found. */
-    OCSD_ERR_RDR_INVALID_INIT,     /**< source reader - invalid initialisation parameter. */
-    OCSD_ERR_RDR_NO_DECODER,       /**< source reader - not trace decoder set. */
+    OCSD_ERR_RDR_FILE_NOT_FOUND,   /**< 13 source reader - file not found. */
+    OCSD_ERR_RDR_INVALID_INIT,     /**< 14 source reader - invalid initialisation parameter. */
+    OCSD_ERR_RDR_NO_DECODER,       /**< 15 source reader - not trace decoder set. */
     /* data path errors */
-    OCSD_ERR_DATA_DECODE_FATAL,    /**< A decoder in the data path has returned a fatal error. */
+    OCSD_ERR_DATA_DECODE_FATAL,    /**< 16 A decoder in the data path has returned a fatal error. */
     /* frame deformatter errors */
-    OCSD_ERR_DFMTR_NOTCONTTRACE,    /**< Trace input to deformatter none-continuous */
-    OCSD_ERR_DFMTR_BAD_FHSYNC,      /**< Bad frame or half frame sync in trace deformatter */
+    OCSD_ERR_DFMTR_NOTCONTTRACE,    /**< 17 Trace input to deformatter none-continuous */
+    OCSD_ERR_DFMTR_BAD_FHSYNC,      /**< 18 Bad frame or half frame sync in trace deformatter */
     /* packet processor errors - protocol issues etc */
-    OCSD_ERR_BAD_PACKET_SEQ,        /**< Bad packet sequence */
-    OCSD_ERR_INVALID_PCKT_HDR,      /**< Invalid packet header */
-    OCSD_ERR_PKT_INTERP_FAIL,       /**< Interpreter failed - cannot recover - bad data or sequence */
+    OCSD_ERR_BAD_PACKET_SEQ,        /**< 19 Bad packet sequence */
+    OCSD_ERR_INVALID_PCKT_HDR,      /**< 20 Invalid packet header */
+    OCSD_ERR_PKT_INTERP_FAIL,       /**< 21 Interpreter failed - cannot recover - bad data or sequence */
     /* packet decoder errors */
-    OCSD_ERR_UNSUPPORTED_ISA,          /**< ISA not supported in decoder. */
-    OCSD_ERR_HW_CFG_UNSUPP,            /**< Programmed trace configuration not supported by decoder.*/
-    OCSD_ERR_UNSUPP_DECODE_PKT,        /**< Packet not supported in decoder */
-    OCSD_ERR_BAD_DECODE_PKT,           /**< reserved or unknown packet in decoder. */
-    OCSD_ERR_COMMIT_PKT_OVERRUN,       /**< overrun in commit packet stack - tried to commit more than available */
-    OCSD_ERR_MEM_NACC,                 /**< unable to access required memory address */
-    OCSD_ERR_RET_STACK_OVERFLOW,       /**< internal return stack overflow checks failed - popped more than we pushed. */
+    OCSD_ERR_UNSUPPORTED_ISA,          /**< 22 ISA not supported in decoder. */
+    OCSD_ERR_HW_CFG_UNSUPP,            /**< 23 Programmed trace configuration not supported by decoder.*/
+    OCSD_ERR_UNSUPP_DECODE_PKT,        /**< 24 Packet not supported in decoder */
+    OCSD_ERR_BAD_DECODE_PKT,           /**< 25 reserved or unknown packet in decoder. */
+    OCSD_ERR_COMMIT_PKT_OVERRUN,       /**< 26 overrun in commit packet stack - tried to commit more than available */
+    OCSD_ERR_MEM_NACC,                 /**< 27 unable to access required memory address */
+    OCSD_ERR_RET_STACK_OVERFLOW,       /**< 28 internal return stack overflow checks failed - popped more than we pushed. */
     /* decode tree errors */
-    OCSD_ERR_DCDT_NO_FORMATTER,         /**< No formatter in use - operation not valid. */
+    OCSD_ERR_DCDT_NO_FORMATTER,         /**< 29 No formatter in use - operation not valid. */
     /* target memory access errors */
-    OCSD_ERR_MEM_ACC_OVERLAP,           /**< Attempted to set an overlapping range in memory access map */
-    OCSD_ERR_MEM_ACC_FILE_NOT_FOUND,    /**< Memory access file could not be opened */
-    OCSD_ERR_MEM_ACC_FILE_DIFF_RANGE,   /**< Attempt to re-use the same memory access file for a different address range */
-    OCSD_ERR_MEM_ACC_RANGE_INVALID,     /**< Address range in accessor set to invalid values */
-    OCSD_ERR_MEM_ACC_BAD_LEN,           /**< Memory accessor returned a bad read length value (larger than requested */
+    OCSD_ERR_MEM_ACC_OVERLAP,           /**< 30 Attempted to set an overlapping range in memory access map */
+    OCSD_ERR_MEM_ACC_FILE_NOT_FOUND,    /**< 31 Memory access file could not be opened */
+    OCSD_ERR_MEM_ACC_FILE_DIFF_RANGE,   /**< 32 Attempt to re-use the same memory access file for a different address range */
+    OCSD_ERR_MEM_ACC_RANGE_INVALID,     /**< 33 Address range in accessor set to invalid values */
+    OCSD_ERR_MEM_ACC_BAD_LEN,           /**< 34 Memory accessor returned a bad read length value (larger than requested */
     /* test errors - errors generated only by the test code, not the library */
-    OCSD_ERR_TEST_SNAPSHOT_PARSE,       /**< test snapshot file parse error */
-    OCSD_ERR_TEST_SNAPSHOT_PARSE_INFO,  /**< test snapshot file parse information */
-    OCSD_ERR_TEST_SNAPSHOT_READ,        /**< test snapshot reader error */
-    OCSD_ERR_TEST_SS_TO_DECODER,        /**< test snapshot to decode tree conversion error */
+    OCSD_ERR_TEST_SNAPSHOT_PARSE,       /**< 35 test snapshot file parse error */
+    OCSD_ERR_TEST_SNAPSHOT_PARSE_INFO,  /**< 36 test snapshot file parse information */
+    OCSD_ERR_TEST_SNAPSHOT_READ,        /**< 37 test snapshot reader error */
+    OCSD_ERR_TEST_SS_TO_DECODER,        /**< 38 test snapshot to decode tree conversion error */
     /* decoder registration */
-    OCSD_ERR_DCDREG_NAME_REPEAT,        /**< attempted to register a decoder with the same name as another one */
-    OCSD_ERR_DCDREG_NAME_UNKNOWN,       /**< attempted to find a decoder with a name that is not known in the library */
-    OCSD_ERR_DCDREG_TYPE_UNKNOWN,       /**< attempted to find a decoder with a type that is not known in the library */
-    OCSD_ERR_DCDREG_TOOMANY,            /**< attempted to register too many custom decoders */
+    OCSD_ERR_DCDREG_NAME_REPEAT,        /**< 39 attempted to register a decoder with the same name as another one */
+    OCSD_ERR_DCDREG_NAME_UNKNOWN,       /**< 40 attempted to find a decoder with a name that is not known in the library */
+    OCSD_ERR_DCDREG_TYPE_UNKNOWN,       /**< 41 attempted to find a decoder with a type that is not known in the library */
+    OCSD_ERR_DCDREG_TOOMANY,            /**< 42 attempted to register too many custom decoders */
     /* decoder config */
-    OCSD_ERR_DCD_INTERFACE_UNUSED,      /**< Attempt to connect or use and interface not supported by this decoder. */
+    OCSD_ERR_DCD_INTERFACE_UNUSED,      /**< 43 Attempt to connect or use and interface not supported by this decoder. */
+    /* additional errors */
+    OCSD_ERR_INVALID_OPCODE,            /**< 44 Opcode found while decoding program memory is illegal */
+    OCSD_ERR_I_RANGE_LIMIT_OVERRUN,     /**< 45 An optional limit on consecutive instructions in range during decode has been exceeded. */
     /* end marker*/
     OCSD_ERR_LAST
 } ocsd_err_t;
@@ -338,8 +333,10 @@ typedef enum _ocsd_isa
 */
 typedef enum _ocsd_sec_level
 {
-    ocsd_sec_secure,   /**< Core is in secure state */
-    ocsd_sec_nonsecure /**< Core is in non-secure state */
+    ocsd_sec_secure,    /**< Core is in secure state */
+    ocsd_sec_nonsecure, /**< Core is in non-secure state */
+    ocsd_sec_root,      /**< PE FEAT_RME: Core is in root state. */
+    ocsd_sec_realm,     /**< PE FEAT_RME: Core is in realm state. */
 } ocsd_sec_level ;
 
 /** Exception level type
@@ -429,14 +426,19 @@ typedef struct _ocsd_pe_context {
 /** memory space bitfield enum for available security states and exception levels used 
    when accessing memory. */
 typedef enum _ocsd_mem_space_acc_t {
-    OCSD_MEM_SPACE_EL1S = 0x1, /**<  S EL1/0 */
-    OCSD_MEM_SPACE_EL1N = 0x2, /**< NS EL1/0 */
-    OCSD_MEM_SPACE_EL2 =  0x4, /**< NS EL2   */
-    OCSD_MEM_SPACE_EL3 =  0x8, /**<  S EL3   */
-    OCSD_MEM_SPACE_EL2S = 0x10, /**< S EL2   */
-    OCSD_MEM_SPACE_S =    0x19, /**< Any  S  */
-    OCSD_MEM_SPACE_N =    0x6, /**< Any NS   */
-    OCSD_MEM_SPACE_ANY =  0x1F, /**< Any sec level / EL - live system use current EL + sec state */
+    OCSD_MEM_SPACE_NONE = 0x0,  /**< Mem space unknown / not yet set */
+    OCSD_MEM_SPACE_EL1S = 0x1,  /**< Secure EL1/0 */
+    OCSD_MEM_SPACE_EL1N = 0x2,  /**< Non Secure EL1/0 */
+    OCSD_MEM_SPACE_EL2 =  0x4,  /**< Non Secure EL2   */
+    OCSD_MEM_SPACE_EL3 =  0x8,  /**< Secure EL3   */
+    OCSD_MEM_SPACE_EL2S = 0x10, /**< Secure EL2   */
+    OCSD_MEM_SPACE_EL1R = 0x20, /**< Realm EL1/0 */
+    OCSD_MEM_SPACE_EL2R = 0x40, /**< Realm EL2 */
+    OCSD_MEM_SPACE_ROOT = 0x80, /**< Root */
+    OCSD_MEM_SPACE_S    = 0x19, /**< Any Secure  */
+    OCSD_MEM_SPACE_N    = 0x6,  /**< Any Non Secure   */
+    OCSD_MEM_SPACE_R    = 0x60, /**< Any Realm */    
+    OCSD_MEM_SPACE_ANY  = 0xFF, /**< Any sec level / EL - live system use current EL + sec state */
 } ocsd_mem_space_acc_t;
 
 /**
@@ -524,10 +526,11 @@ typedef struct _ocsd_file_mem_region {
     (common flags share bitfield with pkt processor common flags and create flags)
     @{*/
 
-#define OCSD_OPFLG_PKTDEC_ERROR_BAD_PKTS  0x00000100  /**< throw error on bad packets input (default is to unsync and wait) */
+#define OCSD_OPFLG_PKTDEC_ERROR_BAD_PKTS  0x00000100  /**< throw error on bad packets input (default is to warn) */
+#define OCSD_OPFLG_PKTDEC_HALT_BAD_PKTS   0x00000200  /**< halt decoder on bad packets (default is to log error and continue by resetting decoder and wait for sync */
 
 /** mask to combine all common packet processor operational control flags */
-#define OCSD_OPFLG_PKTDEC_COMMON (OCSD_OPFLG_PKTDEC_ERROR_BAD_PKTS)
+#define OCSD_OPFLG_PKTDEC_COMMON (OCSD_OPFLG_PKTDEC_ERROR_BAD_PKTS | OCSD_OPFLG_PKTDEC_HALT_BAD_PKTS)
 
 /** @}*/
 
@@ -632,6 +635,56 @@ typedef struct _ocsd_swt_info {
 
 /** @}*/
 
+/** @name Demux Statistics 
+ 
+    Contains statistics for the CoreSight frame demultiplexor. 
+
+    Counts total bytes sent to decoders registered against a trace ID, bytes in the input stream that are
+    associated with a trace ID that has no registered decoder, and frame bytes that are not trace data, but
+    are used to decode the frames - ID bytes, sync bytes etc.
+@{*/
+
+typedef struct _ocsd_demux_stats {
+    uint64_t valid_id_bytes;  /**< number of bytes associated with an ID that has a registered decoder */
+    uint64_t no_id_bytes; /**< number of bytes associated with an ID that has no decoder */
+    uint64_t reserved_id_bytes; /**< number of bytes associated with reserved IDs */
+    uint64_t unknown_id_bytes; /**< bytes processed before ID seen in input frames */
+    uint64_t frame_bytes; /**< number of non-data bytes used for frame de-mux - ID bytes, sync etc */    
+} ocsd_demux_stats_t;
+
+/** @}*/
+
+/** @name Decode statistics
+
+    Contains statistics for bytes decoded by the packet decoder, if statistics are supported.
+
+    Stats block instantiated in the base class - derived protocol specific decoder must initialise and
+    use as required.
+
+    The single channel block contains the stats for the requested channel via the API call.
+
+    The global demux block contains the totals for all channels and non-data bytes used in CoreSight
+    frame demux. This block will show identical data for every requested channel via the API.
+
+@{*/
+
+typedef struct _ocsd_decode_stats {
+    uint32_t version;           /**< library version number */
+    uint16_t revision;          /**< revision number - defines the structure version for the stats. */
+   /* single channel block */
+    uint64_t channel_total;     /**< total bytes processed for this channel */
+    uint64_t channel_unsynced;  /**< number of unsynced bytes processed on this channel */
+    uint32_t bad_header_errs;   /**< number of bad packet header errors */
+    uint32_t bad_sequence_errs; /**< number of bad packet sequence errors */
+    
+    ocsd_demux_stats_t demux;   /**< global demux stats block */
+} ocsd_decode_stats_t;
+
+#define OCSD_STATS_REVISION 0x1
+
+/** @}*/
+
+
 /** @}*/
 #endif // ARM_OCSD_IF_TYPES_H_INCLUDED
 
